#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <correlate_square_region.h>
#include <algorithm>
#include <cassert>
#include <fstream>
#include <GL/glut.h>
#include <iomanip>
#include <iterator>
#include <map>
#include <corner.h>
#include <corner_tool.h>
#include <corrcoef.h>
#include <image.h>
#include <image_fileinput.h>
#include <mesa_onscreen.h>
#include <svd_correspondence.h>
#include <sstream>
#include <vector>


#define SKIP 0
#define NUMIMAGES 10

using namespace std;
using namespace mimas;
using namespace boost;

typedef numeric::ublas::vector< float > FloatVector;
typedef numeric::ublas::vector< int > IntVector;

class CorrelationWeightedProximity:
  public binary_function< shared_ptr< IntVector >, shared_ptr< IntVector >,
                          double >
{
 public:
  CorrelationWeightedProximity( shared_ptr< image< int > > aImage1,
                                shared_ptr< image< int > > aImage2,
                                int aAreaDiv2,
                                double aSigma ):
    pixelCorrelation( aImage1, aImage2, aAreaDiv2, corrCoef ),
    sigma( aSigma ) {}
  double operator()( const shared_ptr< IntVector > &a,
                     const shared_ptr< IntVector > &b ) {
    double
      corr = pixelCorrelation( *a, *b ),
      dist = norm_2( FloatVector( *b - *a ) );
    return
      ( ( corr + 1 ) / 2.0 ) *
      ( ( corr + 1 ) / 2.0 ) *
      exp( -dist / sigma );
  }
 protected:
  corrcoef< vector< int > > corrCoef;
  correlate_square_region< int, corrcoef< vector< int > > >
     pixelCorrelation;
  double sigma;
};


class flow_window: x11_window
{
 public:
  flow_window( x11_display *display, int _width, int _height ):
    x11_window( display, _width, _height )
  { im.init( _width, _height ); }
  void update( const image< int > &_image,
               shared_ptr< map< int, shared_ptr< IntVector > > > _tracking ) {
    im = image< rgba< unsigned char > >( _image );
    trackingOld = tracking;
    tracking = _tracking;
    repaint();
  };
 protected:
  virtual void paint_event(void) throw (mimasexception) {

    mesa_onscreen mesa( this );
    mesa.ortho2D();

    glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
    glRasterPos2i( 0, 0 );
    glPixelZoom( 1, -1 );
    glDrawPixels( width, height,
                  GL_BGRA, GL_UNSIGNED_BYTE, im.rawData().data() );  

    rgba< unsigned char > yellow( 255, 255, 0 ), red( 255, 0, 0 );
    
    for ( map< int, shared_ptr< IntVector > >::const_iterator i =
            tracking->begin(); i != tracking->end(); i++ ) {
      
      shared_ptr< IntVector > vectorNew = i->second;
      
      int
        x = (int)(*vectorNew)( 0 ),
        y = (int)(*vectorNew)( 1 );
      
      glBegin( GL_LINES ); {
        
        if ( trackingOld ) {
          mesa.setColor( yellow );
          shared_ptr< IntVector > vector = (*trackingOld)[ i->first ];
          if ( vector ) {
            int
              xold = (int)(*vector)( 0 ),
              yold = (int)(*vector)( 1 );
            glVertex2i( xold, yold );
            glVertex2i( x   , y    );
          };
        };
        
        mesa.setColor( red );

        glVertex2i( x - 1, y );
        glVertex2i( x + 1, y );
        glVertex2i( x, y - 1 );
        glVertex2i( x, y + 1 );
          
      }; glEnd();
      
      ostringstream stream; stream << i->first;
      glPushMatrix(); {
        glTranslated( x + 4, y + 14, 0 );
        glScaled( 0.05, -0.05, 1.0 );
        mesa.text( stream.str() );
      }; glPopMatrix();

    };
      
    mesa.finish();

  };
  image< rgba< unsigned char > > im;
  shared_ptr< map< int, shared_ptr< IntVector > > > trackingOld, tracking;
};

typedef shared_ptr< flow_window > flow_window_ptr;

int main( int argc, char *argv[] )
{
  int retVal = 0;
  glutInit( &argc, argv );
  try {
  MMERROR( argc >= 5, mimasexception, ,
           "Syntax : " << argv[0] << " <sigma> <area2> <corner threshold> "
           "<output filename prefix> files ..." << endl
           << "Example: " << argv[0] << " 60 3 0.9 result input*.pgm" );
  double sigma = atof( argv[ 1 ] );
  int area2 = atoi( argv[ 2 ] );
  double threshold = atof( argv[ 3 ] );

  x11_display display;
  flow_window_ptr window;

  shared_ptr< map< int, shared_ptr< IntVector > > > tracking;
  shared_ptr< image< int > > im;

  int index = 0;
  
  for ( int c=0; c<argc-5; c++ ) {
	 
    shared_ptr< image< int > > imageNew( new image<int> );
    ifstream file( argv[ c+ 5 ], ios::binary ); file >> *imageNew;

    if ( !window )
      window = flow_window_ptr( new flow_window( &display,
                                                 imageNew->getWidth(),
                                                 imageNew->getHeight() ) );
    
    corner_image corners;

    corner_tool<int> cornerTool( threshold );
    cornerTool.filterImage( *imageNew, corners );

    shared_ptr< vector< shared_ptr< IntVector > > > featuresNew
      ( new vector< shared_ptr< IntVector > > );

    for ( int i=0; i<(signed)corners.rawData().num_elements(); i++ ) {
      corner_ptr corner_p = dynamic_pointer_cast< corner >
        ( corners.rawData().data()[i] );
      if ( corner_p )
        featuresNew->
          push_back( shared_ptr< IntVector >
                     ( new IntVector( corner_p->getPosition() ) ) );
    };
      
    // Initialise map for tracking.
    shared_ptr< map< int, shared_ptr< IntVector > > > trackingNew
      ( new map< int, shared_ptr< IntVector > > );
    shared_ptr< vector< shared_ptr< IntVector > > > lostFeatures
      ( new vector< shared_ptr< IntVector > > );

    if ( tracking ) { 
      
      CorrelationWeightedProximity cwp( im, imageNew, area2, sigma );
      svd_correspondence< map< int, shared_ptr< IntVector > >::
        const_iterator, CorrelationWeightedProximity >
        corr( cwp, tracking->begin(), tracking->size() );
      corr.match( featuresNew->begin(), featuresNew->size(),
                  insert_iterator< map< int, shared_ptr< IntVector > > >
                  ( *trackingNew, trackingNew->begin() ),
                  back_insert_iterator< vector< shared_ptr< IntVector > > >
                  ( *lostFeatures ) );

    } else
      // Initially schedule all features for assignment of new indices.
      lostFeatures = featuresNew;

    for ( vector< shared_ptr< IntVector > >::iterator i=lostFeatures->begin();
          i != lostFeatures->end(); i++ )
      trackingNew->insert( pair< int, shared_ptr< IntVector > >
                           ( index++, *i ) );

    window->update( *imageNew, trackingNew );
    
    im = imageNew;
    tracking = trackingNew;

  };
  
  } catch ( exception &e ) {
    cerr << endl << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}
