//
// Binary image morphology tool
//


#ifndef MORPHOLOGY_TOOL_H
#define MORPHOLOGY_TOOL_H

#include <iostream>
#include <cmath>
#include <values.h>
#include "image.h"
#include "rgba.h"

namespace mimas {


/**
 * Morphology tool for binary images
 * @author Bala Amavasai (bala@amavasai.org)
 * @date Tue Feb 18 16:27:41 GMT 2003
 * $Header: /cvs/mimas2/include/binmorph_tool.h,v 1.1.1.1 2005/08/09 15:37:45 engmb Exp $
 */


class binmorph_tool //: public object
{
	private:
		image<int> structElement; ///< the structuring element
		enum operation {binmorph_dilation, binmorph_erosion};
		void morphology(image<int> &imagein, image<int> &imageout, binmorph_tool::operation op );
		int structElementOnes(void); ///< the number of 1's in the structuring element
		void labelConnectedGroup(image<int> &imagein, image<int> &imageout, int x, int y, int label); // private recursive function for connected components

	public:
		binmorph_tool(void); ///< constructor
		~binmorph_tool(void); ///< destructor
		// void setStructElement(image<int> &structElem); ///< redefine the structuring element
		void setSizeStructElement(int size); ///< redefine the size of structuring element (default is a 3x3 square)
		void greyToBinaryOtsu(image<int> &imagein, image<int> &imageout); ///< convert imagein to binary form based on Otsu's optimal method and store in imageout
		void greyToBinary(image<int> &imagein, image<int> &imageout, int threslevel); ///< convert imagein to binary form based on user defined threshold level and store in imageout
		void greyToBinary(image<int> &imagein, int threslevel); ///< convert imagein to binary form based on user defined threshold level
		void binaryToGrey(image<int> &imagein); ///< normalise binary image to the range 0..255
		void normaliseToGrey(image<int> &imagein); ///< normalise imagein to the range 0..255
		void normaliseToColour(image<int> &imagein, image< rgba< int > > &imageout); ///< normalise imagein to colour in the range 0..255*255*255. Useful for displaying connected components
		void negate(image<int> &imagein); ///< negate image so component appears as 1 and background as 0
		image<int> image_and(image<int> &image1, image<int> &image2); ///<pixelwise and of the two images.
		void erosion(image<int> &imagein, image<int> &imageout); ///< erode binary image
		void dilation(image<int> &imagein, image<int> &imageout); ///< dilate binary image
		void closing(image<int> &imagein, image<int> &imageout); ///< closing operation on binary image
		void opening(image<int> &imagein, image<int> &imageout); ///< opening operation on binary image
		void difference(image<int> &imagein1, image<int> &imagein2, image<int> &imageout); ///< set of binary points that belong to imagein1 but not on imagein2 *Note that this is not image subtraction!
		void boundaryExtract(image<int> &imagein, image<int> &imageout); ///< set of points on the boundary of set imagein
		void connectedComponents(image<int> &imagein, image<int> &imageout); ///< perform connected components analysis on imagein and store labels into imageout. Use normaliseToGrey to before displaying imageout.
		void distanceTransform(image<int> &imagein, image<int> &imageout); ///< find the distance transform of imagein and store it in imageout. Ref: David Eberly NNTP message id <2vkc27$mjs@trochlea.cs.unc.edu>

		// supplementary functions
		void floodFill(image<int> &imagein, int x, int y, int newval, int oldval);; // flood fill. Replace oldval with newval starting from point x,y
};

};

#endif


