#ifndef CAMERA_H
#define CAMERA_H

#include "mimasconfig.h"

#ifdef HAVE_LIBMINPACK
#ifdef HAVE_LIBLAPACK
#ifdef CAMERA_CALIBRATION_ENABLE

#include <boost/numeric/ublas/io.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/banded.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/symmetric.hpp>
#include <boost/numeric/ublas/triangular.hpp>
#include <boost/numeric/ublas/storage.hpp>
#include "linalg.h"
#include "nlinalg.h"
#include "mimasexception.h"
#include "primitives.h"

namespace mimas { 
namespace camera_calibration {

  /**
   * @defgroup calibration Camera Calibration
   * This group contains classes to calibrate a camera.
   *
   * @addtogroup calibration
   * @{
   */

  /**
   * This class reprents the picture of a chessboard, and the links between 
   * the object and the image.
   */
  struct Chessboard {
    /// a vector of points, in the coordinates of the chessboard
    std::vector< point_f > object;
    
    /// a vector of points, in the coordinates of the picture (in pixels), 
    /// representing the images of the chessboard points; and in the same order 
    /// as the vector object.
    std::vector< pixel_f > image;
  };

  /**
   * This class represents a camera, and provides some functions to calibrate it.
   *
   * The class can be used with a float or double template, depending on the 
   * precision required in the calculus.
   *
   * Important : because of a limitation of a FORTRAN library used by the calibration
   * algorithm, it's only possible to instanciate one class Camera at the same time.
   *
   * @author Julien Faucher (faucherj@gmail.com)
   */
  class Camera {
    
  public:
    /// Matrix definition
    typedef boost::numeric::ublas::matrix< double, boost::numeric::ublas::column_major > Matrix;
    
    /// Vector definition
    typedef boost::numeric::ublas::vector< double > Vector;

    /// Triangular matrix definition
    typedef boost::numeric::ublas::triangular_matrix 
      < double, boost::numeric::ublas::upper, boost::numeric::ublas::column_major > TriangularMatrix;

    /// Symmetric matrix definition
    typedef boost::numeric::ublas::symmetric_matrix 
      < double, boost::numeric::ublas::upper, boost::numeric::ublas::column_major > SymmetricMatrix;

  private:
    /// intrinsic parameters matrix
    Matrix K, K_opt;

    /// distortion parameters
    Vector disto;

    /// set to true if the maximum likelyhood optimisation has worked
    bool optimisation;

    std::vector< Chessboard > chessb;
    std::vector< Matrix > extrinsics;

  protected:
    /**
     * Find all the homographies
     * 
     * Given 4 points and their images for some pictures, this function
     * calculates all the homographies (one by picture).
     */
    static void findHomographies(std::vector< Matrix > &homographies, std::vector< Chessboard > &chess);

    /**
     * Find a first approximation of the intrinsic matrix.
     *
     * This function returns a first approximation of the intrinsics, in the ideal
     * case where there is no noise in the pictures, and no distortion.
     */
    static Matrix initCalibration(std::vector< Matrix > &homographies, std::vector< Chessboard > &chess) 
      throw (mimasexception);

    /**
     * Find the extrinsic parameters.
     *
     * This functions finds the extrinsic parameters (i.e. the rotation and the
     * translation) between each image of the chessboard and the chessboard itself.
     */
    static void findExtrinsics(std::vector< Matrix > &extrinsics, std::vector< Matrix > &homographies, 
			       Matrix &intrinsics, int nbPictures);

    /* void initDistortion(void); */

    /**
     * This function applies the Levenberg-Marquardt algorithm to the first
     * approximation of the intrinsic matrix.
     */
    void maxLikelyhood(Matrix &intrinsics, int nbPictures) throw (mimasexception);

  public:

    /// Constructor : instanciate ONE class
    Camera() throw(mimasexception);

    /// Free the class
    ~Camera();

    /**
     * Calibrate the camera.
     * 
     * This is the function one should call to get the calibration parameters.
     *
     * To calibrate the camera, one should provide four points of the chessboard 
     * in the coordinate of the chessboard (the unit is the square), and the 
     * coordinates of the same points in the picture of the chessboard taken by
     * the camera (the unit is the pixel).
     *
     * A minimum of 4 images is required to compute the calibration, but you will
     * need much more to get an accurate result.
     *
     * @param chess a vector of Chessboard classes, to know the images of the
     *        chessboard points by the camera.
     */
    void calibrate(std::vector< Chessboard > &chess) throw (mimasexception);

    /**
     * Calculate the projection of the chessboard points according to the given values
     * of the intrinsic matrix and the distortion coefficients, and return an array
     * containing the error between the mesured point and the calculated one.
     *
     * This function is used by the MINPACK library (in FORTRAN) to minimize the error
     * using the Levenberg-Marquardt algorithm.
     *
     * @param m the number of functions ie 4 * nbPictures (there are 4 points per
     *        picture)
     * @param x an arrray containing the 5 intrinsic parameters and the 2 distortion
     *        parameters
     * @param fvec an output array, containing the result of each function.
     */
    static void
      fcn(int *m, int */*n*/, double *x, double *fvec, int */*iflag*/);

    /**
     * Find the homography 
     *
     * This function finds the homography between a set of $nb$ points and $nb$ images.
     *
     * @param chess a Chessboard class
     * @param homography a 3*3 matrix, must be initialised.
     */
    static void 
      findHomography(const Chessboard &chess, Matrix &homography);
    
    /// Returns first approximation of the matrix of the intrinsic parameters.
    Matrix getIntrinsicsInit() const { return K; }

    /// Returns the optimized matrix of the intrinsic parameters.
    Matrix getIntrinsics() const { return K_opt; }  

    /// Returns the distortion.
    Vector getDistortion() const { return disto; }

    /// Returns if the optimisation algorithm worked or not.
    bool isOptimized() const { return optimisation; }

  };

  /// pointer to the current class. So DO NOT instanciate more than one class
  /// Camera or the result WILL NOT be correct.
  static Camera *current_cam = NULL;


  /** @} */
    
}
}

#endif
#endif
#endif

#endif /* CAMERA_H */
