#ifndef MESA_OFFSCREEN_H
#define MESA_OFFSCREEN_H

#include "mimasconfig.h"

#ifdef HAVE_OPENGL

#include <boost/shared_ptr.hpp>
#include "image.h"
#include "mesa_base.h"
#include "rgba.h"

#ifndef HAVE_OSMESA
#define GLX_GLXEXT_PROTOTYPES
#include <GL/glx.h>
#else
#include <GL/gl.h>
#ifndef GLAPIENTRY
#define GLAPIENTRY
#endif
#include <GL/osmesa.h>
#endif

namespace mimas {

/** Mesa off-screen rendering.
    Support for off-screen rendering using Mesa.

    No stencil- and accumulation-buffer are allocated.
    
    Do not forget to initialise <EM>glut</EM> at the beginning of your
    program:
    \verbatim
    glutInit( &argc, argv );
    \endverbatim

    If you experience problems, you may want to try:
    \verbatim
    export LIBGL_ALWAYS_INDIRECT=1
    \endverbatim
    This disables hardware rendering.

    If it works:
    - the problem is likely with the display driver
    - it will be very slow

    If it doesn't:
    - there is a different bug, we need to fix

    References:
    \li <A HREF="http://www.mesa3d.org/brianp/sig97/offscrn.htm">http://www.mesa3d.org/brianp/sig97/offscrn.htm</A>
    @author Jan Wedekind (jan@wedesoft.de)
    @date Wed Mar 30 19:35:05 UTC 2005 */
template< typename T >
class mesa_offscreen: public mesa_base
{
 public:
  /** Constructor.
      @param _display X11 display
      @param image Image to draw on.
      @param depthBuffer Boolean indicating, wether to use a depth-buffer. */
  mesa_offscreen( x11_display *_display, image< T > &image,
		  bool depthBuffer = false );
  /** Destructor.
      The destructor just calls the method \c finish() .
      @see finish */
  virtual ~mesa_offscreen(void);
  /** Finish output on the image.
      This method
      \li calls \c glFinish()
      \li copies back the image-data, if \c HAVE_OSMESA is not defined.
      \li destructs all the resources related to off-screen rendering.
      \li finally calls \c mesa_base::finish .
      @see mesa_base::finish */
  virtual void finish(void);
  /** Switch to two-dimensional drawing mode.
      @see mesa_base::ortho2D */
  virtual void ortho2D( int x1, int y1, int x2, int y2 ) const;
  /** Switch to two-dimensional drawing mode.
      @see mesa_base::ortho2D */
  virtual void ortho2D(void) const;
 protected:
  ///
 GLenum getFormat(void) const throw (mimasexception);
  ///
  void setupColorScale(void) const;
#ifndef HAVE_OSMESA
  /// X11 pixmap.
  Pixmap pixmap;
  /// Corresponding GLX pixmap
  GLXPixmap glxPixmap;
  /// GLX context for off-screen rendering.
  GLXContext context;
  /// Pointer to image-data to write back later.
  void *data;
#else
  /// Context for (direct) off-screen rendering using OSMesa.
  OSMesaContext context;
#endif
  /// Width of image.
  int width;
  /// Height of image.
  int height;
  ///
  bool finishOffscreen;
};

};

#include "mesa_offscreen.tcc"

#endif

#endif

