// $Header: /cvs/mimas2/include/pol_locator_hough.h,v 1.1.1.1 2005/08/09 15:37:45 engmb Exp $
#ifndef POL_LOCATOR_HOUGH_H
#define POL_LOCATOR_HOUGH_H

#include "mimasconfig.h"
#include <boost/numeric/ublas/vector.hpp>
#include <cmath>
#include "image.h"
#include "angle.h"
#include "pol_locator.h"

#define MAXVAL	0x7fff

namespace mimas {
class pol_model;
class pol_match;

/**
 * @defgroup houghTransform Hough Transformation
 * A special hough transform 'developed' for the pairs of lines tool.
 * It enables both estimates of the positions of the models *and* the
 * orientations of the models to be found using a single hough transform.
 *
 * There's no clever theories involved its just an idea which seems to
 * work ok. It is used by pol_scene and is not likely to be of much use
 * in other circumstances.
 *
 * I can't call it a hough though because it doesn't inherit from the
 * hough base class (don't want \<double\> pixels)...
 *
 * Maybe it should do in the future , but because it is only used for the
 * POL tool, this is ok for now.
 * @author Stuart Meikle (stu@stumeikle.org)
 * @date Thu Oct 24 19:56:40 CEST 2002
 * @{ */

/** Atomic element of hough transform.
 * small structure used at each pixel in the pol_locator_hough to store an
 * estimate of the angle of the model whose centroid is at that position
 * (has 2 degrees of freedom, essentially, magnitude and direction
 */
class	pol_angle
{
    public:
    int16_t	x,y;	///< x,y being like 'x' and 'y' of a vector pointing in the direction of the x axis of the model

    pol_angle(int a)
    {
	x=y=(int16_t) a;
    }

    pol_angle(double v)
    {
	x=y=(int16_t) v;
    }

    pol_angle()
    {
	x=y=(int16_t) 0;
    }

    operator	int()
    {
	return (int)(x+y);
    }

    operator	double()
    {
	return (double)(x+y);
    }


    int	mag()
    {
	double	dx=(double)x,dy=(double)y;
	return (int)sqrt(dx*dx+dy*dy);
    }

    int	magSquared()
    {
	int	ix=(int)x;
	int	iy=(int)y;
	return (ix*ix+iy*iy);
    }
};


/** The pol locator class.
 * An image of pol_angle pixels. Essentially a hough transform used to assimilate estimates of
 * the possible position and orientation of a model in a scene. */
class	pol_locator_hough : public pol_locator, public image<pol_angle>
{
    protected:
	image<double>	*bell_curve;///< gauss curve. used to speed up plotting entries.
	double			thres;	    ///< used to separate 'peaks' from the background noise. value depends on number of lines in the model
	double			ox,oy;

    public:
    typedef boost::numeric::ublas::vector< double > Vector;
	pol_locator_hough():
   bell_curve(0), thres(0), ox(0), oy(0)
	{}
	pol_locator_hough(double t):
   bell_curve(0), thres(t), ox(0), oy(0)
  {}

	~pol_locator_hough()
	{
	    if (bell_curve)	delete bell_curve;
	}

protected:
	/**
	 * as in <em>hough_transform.cc</em>. used to calculate the size of
	 * the gaussian dist. required to represent a curve with sigma 'sigma'
	 * to the accuracy given
	 */
	int	calcGaussBoxSize( double sigma, double accuracy );
	/**
	 * make an entry at position x,y with weight w and angle a
	 * ca = w*cos(a) and sa = w*sin(a)
	 *
	 */
	void	setLocation( int x, int y, double ca, double sa );

	/**
	 * in order to plot gauss points faster, we calculate the
	 * required distribution first. this assumes that all points will
	 * be equally accurate, but it's fast.
	 */
	void	initFastGauss( double sigma, double precision );

	/**
	 * plot a point as position posn with weight 'w' and angle 'angle'
	 * replaced by addEntry now 200212
	 */
	void	addGaussPointFast( const Vector &posn, double w, double angle );

public:
	/*
	 * display the hough transform, i.e. for debugging
	 *
	 */
	// virtual void display( void );

	/**
	 * Provide the standard interface defined functions
	 *
	 */
	virtual void			init( int numdims, ...);
	virtual void			addEntry( double weight, ... );
	virtual pol_match_ptr_list getMatches();
	void				setOffset( double dx, double dy )
					{ ox=dx; oy=dy; }

};

///@}

}

#endif

