#ifndef PYRAMID_TOOL_H
#define PYRAMID_TOOL_H

#include <iostream>
#include <vector>
#include <cmath>
#include <values.h>
#include "image.h"
#include "image_funcs.h"

namespace mimas {
/**
 * Pyramid tool for creating Gaussian and Laplacian pyramids
 * @author Bala Amavasai (bala@amavasai.org)
 * @date Mon Jul 15 11:39:38 UTC 2002
 * $Header: /cvs/mimas2/include/pyramid_tool.h,v 1.1.1.1 2005/08/09 15:37:45 engmb Exp $
 */
class pyramid_tool
{
	private:
		double a,b,c; ///< Note that these are prespecified values given in Burt & Adelson's 1983 paper
		
	public:
		pyramid_tool(void); ///< constructor
		~pyramid_tool(void); ///< destructor
		void reduceGaussian(image<double> &imagein, image<double> &imageout);
		void reduceGaussian(image<double> &imagein, vector < image<double> > &vectimageout, int numlevels);
		void expandGaussian(image<double> &imagein, image<double> &imageout);
		void expandGaussian(image<double> &imagein, vector < image<double> > &vectimageout, int numlevels);
		void buildLaplacian(image<double> &imagein, vector < image<double> > &vectimageout, int numlevels);
		void reconstructLaplacian(vector < image<double> > &vectimagein, image<double> &imageout);

 
};

pyramid_tool::pyramid_tool()
{
	a=0.4;
	b=0.25;
	c=b-a/2;
}

pyramid_tool::~pyramid_tool()
{
}


void pyramid_tool::buildLaplacian(image<double> &imagein, vector < image<double> > &vectimageout, int numlevels)
{
	image<double> image;
	image<double> rimage, eimage;
	
	vectimageout.clear();
	for (int r=0; r<numlevels; r++)
		vectimageout.insert(vectimageout.begin(),image);

	image=imagein;
	
	for (int r=0; r<numlevels-1; r++)
	{
		reduceGaussian(image, rimage);
		expandGaussian(rimage,eimage);

		vectimageout[r].init(image.getWidth(),image.getHeight());

		for (int j=0; j<image.getHeight(); j++)
			for (int i=0; i<image.getWidth(); i++)
           vectimageout[r].pixel(i,j)=image.pixel(i,j)-eimage.pixel(i,j);
		
		image.clear();
		image=rimage;
	}
	vectimageout[numlevels-1]=image; // at the lowest level the Laplacian image is the final reduceGaussiand image;

}

void pyramid_tool::reconstructLaplacian(vector < image<double> > &vectimagein, image<double> &imageout)
{
	int numlevels=vectimagein.size();
	
	if (numlevels<=0) return;

	imageout=vectimagein[numlevels-1];
	
	for (int r=numlevels-2; r>=0; r--)
	{
		image<double> eimage;

		expandGaussian(imageout,eimage);

		imageout.init(eimage.getWidth(),eimage.getHeight());
		
		for (int j=0; j<imageout.getHeight(); j++)
			for (int i=0; i<imageout.getWidth(); i++)
           imageout.pixel(i,j) = eimage.pixel(i,j)+vectimagein[r].pixel(i,j);
			
	}
}


void pyramid_tool::reduceGaussian(image<double> &imagein, vector < image<double> > &vectimageout, int numlevels)
{
	image<double> image;
	
	vectimageout.clear();
	for (int r=0; r<numlevels; r++)
		vectimageout.insert(vectimageout.begin(),image);

	image=imagein;
	
	if (numlevels>=1)
		vectimageout[0]=imagein;

	for (int r=1; r<numlevels; r++)
	{
		image<double> tmpimage;
		//tmpimage=new image<double>;

		reduceGaussian(image,vectimageout[r]);
		
		image.clear();
		image=vectimageout[r];
	}

}

void pyramid_tool::expandGaussian(image<double> &imagein, vector < image<double> > &vectimageout, int numlevels)
{
	image<double> image;
	
	vectimageout.clear();
	for (int r=0; r<numlevels; r++)
		vectimageout.insert(vectimageout.begin(),image);

	image=imagein;
	
	if (numlevels>=1)
		vectimageout[0]=imagein;

	for (int r=1; r<numlevels; r++)
	{
		image<double> tmpimage;
		//tmpimage=new image<double>;

		expandGaussian(image,vectimageout[r]);
		
		image.clear();
		image=vectimageout[r];
	}

}

void pyramid_tool::reduceGaussian(image<double> &imagein, image<double> &imageout)
{
	imageout.init(imagein.getWidth()/2, imagein.getHeight()/2);

	double gkernel[5][5]={  {c*c, b*c, a*c, b*c, c*c},
							{c*b, b*b, a*b, b*b, c*b},
							{c*a, b*a, a*a, b*a, c*a},
							{c*b, b*b, a*b, b*b, c*b},
							{c*c, b*c, a*c, b*c, c*c} };


							
    double sumgkernel=0.0;
	for (int i=0; i<5; i++)
		for (int j=0; j<5; j++)
			sumgkernel+=gkernel[i][j];

   cerr << "sumgkernel = " << sumgkernel << std::endl;
	
	for (int j=0; j<imageout.getHeight(); j++)	
		for (int i=0; i<imageout.getWidth(); i++)	
		{
			double intensity=0.0;
			for (int y=-2; y<=2; y++)
				for (int x=-2; x<=2; x++)
					intensity+=gkernel[x+2][y+2]*imagein.getPixel(2*i+x,2*j+y);

			imageout.pixel(i,j) = intensity / sumgkernel;
		}
}

void pyramid_tool::expandGaussian(image<double> &imagein, image<double> &imageout)
{
	imageout.init(imagein.getWidth()*2, imagein.getHeight()*2);

	double gkernel[5][5]={  {c*c, b*c, a*c, b*c, c*c},
							{c*b, b*b, a*b, b*b, c*b},
							{c*a, b*a, a*a, b*a, c*a},
							{c*b, b*b, a*b, b*b, c*b},
							{c*c, b*c, a*c, b*c, c*c} };


							
    double sumgkernel=0.0;
	for (int i=0; i<5; i++)
		for (int j=0; j<5; j++)
			sumgkernel+=gkernel[i][j];

   cerr << "sumgkernel = " << sumgkernel << std::endl;
	
	for (int j=0; j<imageout.getHeight(); j++)	
		for (int i=0; i<imageout.getWidth(); i++)	
		{
			double intensity=0.0;
			for (int y=-2; y<=2; y++)
				for (int x=-2; x<=2; x++)
					intensity+=gkernel[x+2][y+2]*imagein.getPixel((i-x)/2,(j-y)/2);

			imageout.pixel(i,j) = intensity / sumgkernel;
		}
}
}
#endif
