#ifndef __VECTORFIELD_H
#define __VECTORFIELD_H

#include <boost/numeric/ublas/vector.hpp>
#include <boost/multi_array.hpp>
#include <functional>

namespace mimas {

/** @ingroup warping
    Precomputed tensor.
    This class takes samples \f$t((x_1,x_2)^\top-\vec{m})\f$ of a tensor and
    stores them in a vector field
    (\f$\vec{m}=(\frac{\mathrm{width}}{2},\frac{\mathrm{height}}{2})\f$,
    \f$(x_1,x_2)\in\{(0,0),(1,0),\ldots,
    (\mathrm{width}-1,\mathrm{height}-1)\}\f$.

    Using a precomputed field for warping several images, instead of using the
    same tensor function a couple of times, usually is faster but also more
    memory-consuming.
    @todo Use integer vectors
    @author Jan Wedekind (jan@wedesoft.de)
    @date Wed Jul 07 14:33:00 UTC 2004
    @see warp
    @todo Replace with boost::multi_array */
template< class Tensor >
class vectorfield: public std::unary_function
   < boost::numeric::ublas::vector< double >,
     boost::numeric::ublas::vector< double > >
{
 public:
  typedef boost::numeric::ublas::vector< double > Vector;
  /** Constructor.
      @param width Width of vector field.
      @param height Height of vector field.
      @param aTensor Tensor to take samples from. */
  vectorfield( int width, int height, const Tensor &aTensor );
  /// Function operator.
  Vector operator()( const Vector &v ) const {
    int
      height = data.shape()[ 0 ],
      width = data.shape()[ 1 ],
      x = (int)rint( v( 0 ) + width * 0.5 ),
      y = (int)rint( v( 1 ) + height * 0.5 );
    assert( x >= 0 && x < width );
    assert( y >= 0 && y < height );
    return data[ y ][ x ];
  };
 protected:
  /// The samples are stored in this array.
  boost::multi_array< Vector, 2 > data;
};

};

#include "vectorfield.tcc"

#endif
