#include "mimasconfig.h"

#ifdef HAVE_OPENGL

#ifndef NDEBUG
#include <iostream>
#endif
#include <X11/keysym.h>
#include "x11_display.h"
#include "x11_window.h"

using namespace std;

namespace mimas {

  // x11_display_ptr x11_window::display;

  // bool x11_window::state = true;

// List of windows.
// std::map< Window, std::pair< void (*)(void *), void * > >
//  x11_window::windows;

x11_window::x11_window( x11_display *_display,
			int _width, int _height,
			bool depthBuffer ):
  display(_display), visualInfo(NULL), width(_width), height(_height)
{
  // state = true;
  MMERROR( width > 0 && height > 0, mimasexception, ,
           width << 'x' << height << " is an illegal image-size." );
  
  // Request desired X11 visual.
  int attribs[] = { GLX_RGBA,
		    GLX_RED_SIZE, 1, GLX_GREEN_SIZE, 1, GLX_BLUE_SIZE, 1,
		    GLX_DEPTH_SIZE, depthBuffer ? 16 : 0, None };
  visualInfo = glXChooseVisual( display->get(),
				DefaultScreen( display->get() ),
				attribs );
  MMERROR( visualInfo != NULL, mimasexception, ,
	   "Error choosing desired X11 visual." );

  try {
      
    // Create a color map.
    colorMap = XCreateColormap( display->get(),
				RootWindow( display->get(),
					    visualInfo->screen ),
				visualInfo->visual, AllocNone );
    MMERROR( colorMap != 0, mimasexception, ,
	     "Couldn't allocate X11 colormap." );
        
    try {
          
      XSetWindowAttributes attributes;
      attributes.colormap = colorMap;
      attributes.border_pixel = 0;
      attributes.event_mask =
	ExposureMask | StructureNotifyMask | KeyPressMask;

#ifndef NDEBUG
      cerr << "Creating window of size " << _width << 'x' << _height << endl;
#endif
          
      window = XCreateWindow( display->get(),
			      RootWindow( display->get(),
					  visualInfo->screen), 0,
			      0, width, height,
			      0, visualInfo->depth, InputOutput,
			      visualInfo->visual,
			      CWBorderPixel | CWColormap | CWEventMask,
			      &attributes );
          
      MMERROR( window != 0, mimasexception, , "Error creating X11 window." );

      try {
            
	// Add window to list of display-class.
	display->add_window( this );

	// Wait for X-server.
	XEvent event;
	XMapWindow( display->get(), window );
	XIfEvent( display->get(), &event, waitForNotify, (char *)window );

      } catch ( mimasexception &e ) {

	display->remove_window( this );

	XDestroyWindow( display->get(), window );
	throw e;
        
      };

    } catch ( mimasexception &e ) {
      
      XFreeColormap( display->get(), colorMap );
      throw e;
      
    };
        
  } catch ( mimasexception &e ) {
    
    XFree( visualInfo );
    throw e;
    
  };

}

x11_window::~x11_window(void)
{
  display->remove_window( this );
  XDestroyWindow( display->get(), window );
  XFreeColormap( display->get(), colorMap );
  XFree( visualInfo );
}

Bool x11_window::waitForNotify( Display *, XEvent *e, char *arg )
{
  return ( e->type == MapNotify ) && ( e->xmap.window == (Window)arg );
}

void x11_window::resize( int _width, int _height )
{
  if ( width != _width || height != _height ) {
#ifndef NDEBUG
    cerr << "Resizing to " << _width << "x" << _height << endl;
#endif
    XResizeWindow( display->get(), window, _width, _height );
    width = _width;
    height = _height;
    XFlush( display->get() );
  };
}

void x11_window::repaint(void)
{
  paint_event();
}

void x11_window::handle_event( XEvent &event ) throw (mimasexception)
{
  switch ( event.type ) {
  case ConfigureNotify:
    while ( XCheckTypedWindowEvent( display->get(), window, ConfigureNotify,
				    &event ) );
    paint_event();
    break;
  case Expose:
    while ( XCheckTypedWindowEvent( display->get(), window, Expose, &event ) );
    paint_event();
    break;
  case KeyPress:
    key_event( event.xkey );
    break;
  default:
    break;
  };
}

void x11_window::key_event( XKeyEvent &xkey ) throw (mimasexception)
{
  switch ( xkey.keycode ) {
  case 0x09: // escape
  case 0x41: // space
    display->quit();
    break;
  default:
    break;
  };
}

};

#endif

